#!/usr/bin/env python

"A simple file indexer."

import codecs

class Parser:
    def __init__(self, dict_location, encoding=None):
        self.dict_location = dict_location
        self.encoding = encoding

    def _get_file_content(self):
        if self.encoding is None:
            f = open(self.dict_location)
        else:
            f = codecs.open(self.dict_location, encoding=self.encoding)
        s = f.read()
        f.close()
        return s

    def send_entries(self, channel):

        "Send word entries from the file."

        tokens = self._get_file_content().split()
        index = {}

        words = []
        for token in tokens:
            if token not in words:
                channel.send((token, self.dict_location))
                words.append(token)

class Indexer:
    def __init__(self):
        self.index = {}

    def get_index(self):
        return self.index

    def add_entry(self, entry):

        "Add the given word 'entry' (token, dict_location) to the index."

        token, dict_location = entry
        slot = self.index
        for c in token:
            if not slot.has_key(c):
                slot[c] = {}, []
            slot, words = slot[c]

        if token not in words:
            words.append(token)

class Searcher:
    def __init__(self, index):
        self.index = index

    def find(self, pattern):

        "Find words beginning with the given 'pattern'."

        slot = self.index
        words = []

        for c in pattern:
            if not slot.has_key(c):
                return []
            slot, words = slot[c]

        results = []
        results += words
        results += self.get_all_words(slot)
        return results

    def get_all_words(self, slot):

        "Get all words under the given index 'slot'."

        all_words = []
        keys = slot.keys()
        keys.sort()
        for c in keys:
            this_slot, words = slot[c]
            all_words += words
            all_words += self.get_all_words(this_slot)
        return all_words

# vim: tabstop=4 expandtab shiftwidth=4
